# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 Avencall
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

import unittest

from mock import Mock, patch
from hamcrest import assert_that, contains_inanyorder, equal_to
import time

from xivo_dao.alchemy.session import Session
from xivo_dao.alchemy.user import User

from xivo_auth.plugins import backends
from xivo_auth.plugins.backends.default_acls import DEFAULT_USER_ACLS


class TestVerifyPassword(unittest.TestCase):

    @patch('xivo_auth.plugins.backends.xivo_session.session_dao.get_session')
    def test_to_get_valid_session(self, session_dao_mock):
        future_time = int(time.time()) + 86400
        serialized_data = '_USR|O:8:"stdClass":2:{s:5:"login";s:4:"root";s:6:"passwd";s:9:"superpass";}_WEBI|'
        session_mock = Mock(expire=future_time, data=serialized_data)

        session_dao_mock.return_value = session_mock

        args = {'cookie': '1234'}

        backend = backends.XiVOSession('config')

        result = backend.verify_password(None, None, args)

        assert_that(result, equal_to(True))
        session_dao_mock.assert_called_once_with('1234')

    @patch('xivo_auth.plugins.backends.xivo_session.session_dao.get_session')
    def test_to_get_valid_session_without_login(self, session_dao_mock):
        future_time = int(time.time()) + 86400
        serialized_data = '_USR|O:8:"stdClass":2:{s:5:"notexpected";s:4:"invalidvalue";s:6:"passwd";s:9:"superpass";}_WEBI|'
        session_mock = Mock(expire=future_time, data=serialized_data)

        session_dao_mock.return_value = session_mock

        args = {'cookie': '1234'}

        backend = backends.XiVOSession('config')

        result = backend.verify_password(None, None, args)

        assert_that(result, equal_to(False))
        session_dao_mock.assert_called_once_with('1234')

    @patch('xivo_auth.plugins.backends.xivo_session.session_dao.get_session')
    def test_to_handle_invalid_session(self, session_dao_mock):
        future_time = int(time.time()) + 86400
        serialized_data = 'ERROR|O:8:"stdClass":2:{s:5:"login";s:4:"root";s:6:"passwd";s:9:"superpass";}_WEBI|'
        session_mock = Mock(expire=future_time, data=serialized_data)

        session_dao_mock.return_value = session_mock

        args = {'cookie': '1234'}

        backend = backends.XiVOSession('config')

        result = backend.verify_password(None, None, args)

        assert_that(result, equal_to(False))
        session_dao_mock.assert_called_once_with('1234')

    @patch('xivo_auth.plugins.backends.xivo_session.session_dao.get_session')
    @patch('xivo_auth.plugins.backends.xivo_session.admin_dao.get_admin_id')
    def test_to_get_root_user_ids(self, admin_dao_mock, session_dao_mock):
        future_time = int(time.time()) + 86400
        serialized_data = '_USR|O:8:"stdClass":2:{s:5:"login";s:4:"root";s:6:"passwd";s:9:"superpass";}_WEBI|'
        session_mock = Mock(expire=future_time, data=serialized_data)

        session_dao_mock.return_value = session_mock
        admin_dao_mock.return_value = 'foo-uuid'

        args = {'cookie': '1234'}

        backend = backends.XiVOSession('config')

        result = backend.get_ids(None, args)

        assert_that(result, equal_to(('foo-uuid', None)))
        session_dao_mock.assert_called_once_with('1234')

    @patch('xivo_auth.plugins.backends.xivo_session.session_dao.get_session')
    @patch('xivo_auth.plugins.backends.xivo_session.admin_dao.get_admin_id')
    @patch('xivo_auth.plugins.backends.xivo_session.user_dao.get_by')
    def test_to_get_user_ids(self, user_dao_mock, admin_dao_mock, session_dao_mock):
        future_time = int(time.time()) + 86400
        serialized_data = '_USR|O:8:"stdClass":2:{s:5:"login";s:4:"user";s:6:"passwd";s:9:"superpass";}_WEBI|'
        session_mock = Mock(expire=future_time, data=serialized_data)

        session_dao_mock.return_value = session_mock
        admin_dao_mock.return_value = '123'
        user_dao_mock.return_value = Mock(User, uuid='foo-uuid')

        args = {'cookie': '1234'}

        backend = backends.XiVOSession('config')

        result = backend.get_ids(None, args)

        assert_that(result, equal_to(('foo-uuid', 'foo-uuid')))
        session_dao_mock.assert_called_once_with('1234')

    @patch('xivo_auth.plugins.backends.xivo_session.session_dao.get_session')
    def test_to_get_root_user_acls(self, session_dao_mock):
        future_time = int(time.time()) + 86400
        serialized_data = '_USR|O:8:"stdClass":2:{s:5:"login";s:4:"root";s:6:"passwd";s:9:"superpass";}_WEBI|'
        session_mock = Mock(expire=future_time, data=serialized_data)

        session_dao_mock.return_value = session_mock

        args = {'cookie': '1234'}

        backend = backends.XiVOSession('config')

        result = backend.get_acls(None, args)

        assert_that(result, equal_to(['confd.#']))
        session_dao_mock.assert_called_once_with('1234')

    @patch('xivo_auth.plugins.backends.xivo_session.session_dao.get_session')
    def test_to_get_user_acls(self, session_dao_mock):
        future_time = int(time.time()) + 86400
        serialized_data = '_USR|O:8:"stdClass":2:{s:5:"login";s:4:"user";s:6:"passwd";s:9:"superpass";}_WEBI|'
        session_mock = Mock(expire=future_time, data=serialized_data)

        session_dao_mock.return_value = session_mock

        args = {'cookie': '1234'}

        backend = backends.XiVOSession('config')

        result = backend.get_acls(None, args)

        assert_that(result, equal_to(DEFAULT_USER_ACLS))
        session_dao_mock.assert_called_once_with('1234')
